@@ -3,7 +3,7 @@
  *  Copyright (c) 2006 Sam Hocevar <sam@zoy.org>
  *                All Rights Reserved
  *
- *  $Id: debug.c 307 2007-11-02 21:40:38Z sam $
+ *  $Id$
  *
  *  This program is free software. It comes without any warranty, to
  *  the extent permitted by applicable law. You can redistribute it
@@ -38,6 +38,10 @@
 
 extern int _zz_debugfd;
 
+/**
+ * Helper macro to write an integer value to a given file descriptor,
+ * either in base 10 or in hexadecimal.
+ */
 #define WRITE_INT(fd, i, base) \
     do \
     { \
@@ -52,6 +56,17 @@ extern int _zz_debugfd;
         write(fd, b + 1, (int)(buf + 127 - b)); \
     } while(0)
 
+/**
+ * Format a string, printf-like, and write the resulting data to zzuf's
+ * debug file descriptor _zz_debugfd. If the debug file descriptor is
+ * still -1, this function does nothing.
+ *
+ * This function's code is roughly equivalent to the following *printf
+ * calls, except it only uses signal-safe functions:
+ *  - fprintf(stderr, "** zzuf debug ** ");
+ *  - vfprintf(stderr, format, args);
+ *  - fprintf(stderr, "\n");
+ */
 void _zz_debug(char const *format, ...)
 {
     static char const *hex2char = "0123456789abcdef";
@@ -65,14 +80,6 @@ void _zz_debug(char const *format, ...)
     saved_errno = errno;
     va_start(args, format);
 
-#if 0
-    /* This function's code is equivalent to the following *printf calls,
-     * except it only uses signal-safe functions */
-    fprintf(stderr, "** zzuf debug ** ");
-    vfprintf(stderr, format, args);
-    fprintf(stderr, "\n");
-#endif
-
     write(_zz_debugfd, "** zzuf debug ** ", 17);
     for(f = format; *f; f++)
     {
@@ -126,6 +133,22 @@ void _zz_debug(char const *format, ...)
             WRITE_INT(_zz_debugfd, i, 10);
             f += 2;
         }
+        else if(f[0] == 'g')
+        {
+            double g = va_arg(args, double), h = 0.0000001;
+            int i = g;
+            WRITE_INT(_zz_debugfd, i, 10);
+            for(i = 0; i < 7; i++)
+            {
+                g = (g - (int)g) * 10;
+                h *= 10;
+                if(g < h)
+                    break;
+                if(i == 0)
+                    write(_zz_debugfd, ".", 1);
+                write(_zz_debugfd, hex2char + (int)g, 1);
+            }
+        }
         else if(f[0] == 'p')
         {
             uintptr_t i = va_arg(args, uintptr_t);
