@@ -169,7 +169,7 @@ int main(int argc, char *argv[])
 #   define OPTSTR_RLIMIT_CPU ""
 #endif
 #define OPTSTR "+" OPTSTR_REGEX OPTSTR_RLIMIT_MEM OPTSTR_RLIMIT_CPU \
-                "Ab:B:C:dD:f:F:il:mnp:P:qr:R:s:St:vxhV"
+                "Ab:B:C:dD:f:F:ij:l:mnp:P:qr:R:s:St:vxhV"
 #define MOREINFO "Try `%s --help' for more information.\n"
         int option_index = 0;
         static struct myoption long_options[] =
@@ -188,11 +188,11 @@ int main(int argc, char *argv[])
             { "exclude",     1, NULL, 'E' },
 #endif
             { "fuzzing",     1, NULL, 'f' },
-            { "max-forks",   1, NULL, 'F' },
             { "stdin",       0, NULL, 'i' },
 #if defined HAVE_REGEX_H
             { "include",     1, NULL, 'I' },
 #endif
+            { "jobs",        1, NULL, 'j' },
             { "list",        1, NULL, 'l' },
             { "md5",         0, NULL, 'm' },
             { "max-memory",  1, NULL, 'M' },
@@ -259,9 +259,9 @@ int main(int argc, char *argv[])
         case 'f': /* --fuzzing */
             opts->fuzzing = myoptarg;
             break;
-        case 'F': /* --max-forks */
-            opts->maxchild = atoi(myoptarg) > 1 ? atoi(myoptarg) : 1;
-            break;
+        case 'F':
+            fprintf(stderr, "%s: `-F' is deprecated, use `-j'\n", argv[0]);
+            return EXIT_FAILURE;
         case 'i': /* --stdin */
             setenv("ZZUF_STDIN", "1", 1);
             break;
@@ -277,6 +277,9 @@ int main(int argc, char *argv[])
             }
             break;
 #endif
+        case 'j': /* --jobs */
+            opts->maxchild = atoi(myoptarg) > 1 ? atoi(myoptarg) : 1;
+            break;
         case 'l': /* --list */
             opts->list = myoptarg;
             break;
@@ -313,7 +316,9 @@ int main(int argc, char *argv[])
         case 's': /* --seed */
             tmp = strchr(myoptarg, ':');
             opts->seed = atol(myoptarg);
-            opts->endseed = tmp ? (uint32_t)atoi(tmp + 1) : opts->seed + 1;
+            opts->endseed = tmp ? tmp[1] ? (uint32_t)atoi(tmp + 1)
+                                         : (uint32_t)-1UL
+                                : opts->seed + 1;
             break;
         case 'S': /* --signal */
             setenv("ZZUF_SIGNAL", "1", 1);
@@ -854,30 +859,31 @@ static char const *sig2str(int signum)
 {
     switch(signum)
     {
-        case SIGABRT:  return " (SIGABRT)";
-        case SIGFPE:   return " (SIGFPE)";
-        case SIGILL:   return " (SIGILL)";
 #ifdef SIGQUIT
-        case SIGQUIT:  return " (SIGQUIT)";
+        case SIGQUIT:  return " (SIGQUIT)"; /* 3 */
 #endif
-        case SIGSEGV:  return " (SIGSEGV)";
+        case SIGILL:   return " (SIGILL)";  /* 4 */
 #ifdef SIGTRAP
-        case SIGTRAP:  return " (SIGTRAP)";
-#endif
-#ifdef SIGSYS
-        case SIGSYS:   return " (SIGSYS)";
-#endif
-#ifdef SIGEMT
-        case SIGEMT:   return " (SIGEMT)";
+        case SIGTRAP:  return " (SIGTRAP)"; /* 5 */
 #endif
+        case SIGABRT:  return " (SIGABRT)"; /* 6 */
 #ifdef SIGBUS
-        case SIGBUS:   return " (SIGBUS)";
+        case SIGBUS:   return " (SIGBUS)";  /* 7 */
+#endif
+        case SIGFPE:   return " (SIGFPE)";  /* 8 */
+        case SIGSEGV:  return " (SIGSEGV)"; /* 11 */
+        case SIGPIPE:  return " (SIGPIPE)"; /* 13 */
+#ifdef SIGEMT
+        case SIGEMT:   return " (SIGEMT)";  /* ? */
 #endif
 #ifdef SIGXCPU
-        case SIGXCPU:  return " (SIGXCPU)";
+        case SIGXCPU:  return " (SIGXCPU)"; /* 24 */
 #endif
 #ifdef SIGXFSZ
-        case SIGXFSZ:  return " (SIGXFSZ)";
+        case SIGXFSZ:  return " (SIGXFSZ)"; /* 25 */
+#endif
+#ifdef SIGSYS
+        case SIGSYS:   return " (SIGSYS)";  /* 31 */
 #endif
     }
 
@@ -1157,13 +1163,13 @@ static void usage(void)
 #else
     printf("Usage: zzuf [-AdimnqSvx] [-s seed|-s start:stop] [-r ratio|-r min:max]\n");
 #endif
-    printf("              [-f fuzzing] [-D delay] [-F forks] [-C crashes] [-B bytes]\n");
+    printf("              [-f fuzzing] [-D delay] [-j jobs] [-C crashes] [-B bytes]\n");
     printf("              [-t seconds] ");
 #if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_CPU
     printf(                           "[-T seconds] ");
 #endif
 #if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
-    printf(                                        "[-M megabytes] ");
+    printf(                                        "[-M mebibytes] ");
 #endif
     printf(                                                       "[-b ranges] [-p ports]\n");
     printf("              [-P protect] [-R refuse] [-l list]");
@@ -1190,11 +1196,11 @@ static void usage(void)
     printf("  -E, --exclude <regex>     do not fuzz files matching <regex>\n");
 #endif
     printf("  -f, --fuzzing <mode>      use fuzzing mode <mode> ([xor] set unset)\n");
-    printf("  -F, --max-forks <n>       number of concurrent children (default 1)\n");
     printf("  -i, --stdin               fuzz standard input\n");
 #if defined HAVE_REGEX_H
     printf("  -I, --include <regex>     only fuzz files matching <regex>\n");
 #endif
+    printf("  -j, --jobs <n>            number of simultaneous jobs (default 1)\n");
     printf("  -l, --list <list>         only fuzz Nth descriptor with N in <list>\n");
     printf("  -m, --md5                 compute the output's MD5 hash\n");
 #if defined HAVE_SETRLIMIT && defined ZZUF_RLIMIT_MEM
